@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005 by Rajko Albrecht                                  *
+ *   Copyright (C) 2005-2007 by Rajko Albrecht                             *
  *   ral@alwins-world.de                                                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -23,7 +23,6 @@
 #include "commandline.h"
 #include "kdesvn-config.h"
 #include <kapplication.h>
-#include <dcopclient.h>
 #include <kaboutdata.h>
 #include <kcmdlineargs.h>
 #include <klocale.h>
@@ -38,6 +37,7 @@ static KCmdLineOptions options[] =
 {
     { "r startrev[:endrev]",I18N_NOOP("Execute single subversion command on specific revision(-range)"),0},
     {"R",I18N_NOOP("Ask for revision when executing single command"),0},
+    {"f",I18N_NOOP("Force operation"),0},
     {"o <file>",I18N_NOOP("Save output of subversion command (eg \"cat\") into file <file>"),0},
     {"l <number>",I18N_NOOP("Limit log output to <number>"),0},
     { "+exec <command>",I18N_NOOP("Execute subversion command (\"exec help\" for more information)"),0},
@@ -48,10 +48,10 @@ static KCmdLineOptions options[] =
 int main(int argc, char **argv)
 {
     KAboutData about("kdesvn", I18N_NOOP("kdesvn"), version, description,
-                     KAboutData::License_GPL, "(C) 2005 Rajko Albrecht",0,
+                     KAboutData::License_GPL, "(C) 2005-2007 Rajko Albrecht",0,
                          0, "ral@alwins-world.de");
     about.addAuthor( "Rajko Albrecht", 0, "ral@alwins-world.de" );
-    about.setHomepage("http://www.alwins-world.de/programs/kdesvn/");
+    about.setHomepage("http://www.alwins-world.de/wiki/programs/kdesvn/");
     about.setBugAddress("kdesvn-bugs@alwins-world.de");
 
     KCmdLineArgs::init(argc, argv, &about);
@@ -60,9 +60,6 @@ int main(int argc, char **argv)
 
     KApplication app;
 
-    // register ourselves as a dcop client
-    app.dcopClient()->registerAs(app.name(), false);
-
     // see if we are starting with session management
     if (app.isRestored())
     {
