@@ -1,3 +1,8 @@
+/*
+ * Port for usage with qt-framework and development for kdesvn
+ * (C) 2005-2007 by Rajko Albrecht
+ * http://www.alwins-world.de/wiki/programs/kdesvn
+ */
 /*
  * ====================================================================
  * Copyright (c) 2002-2005 The RapidSvn Group.  All rights reserved.
@@ -31,8 +36,8 @@
 #pragma warning (disable: 4786)
 #endif
 
-// Ignore MSVC 7 compiler warning: C++ exception specification
-#if defined (_MSC_VER) && _MSC_VER > 1200 && _MSC_VER <= 1310
+// Ignore MSVC 7 & 2005 compiler warning: C++ exception specification
+#if defined (_MSC_VER) && _MSC_VER > 1200 && _MSC_VER <= 1410
 #pragma warning (disable: 4290)
 #endif
 
@@ -341,7 +346,8 @@ namespace svn
      * Creates a directory directly in a repository or creates a
      * directory on disk and schedules it for addition. If <i>path</i>
      * is a URL then authentication is usually required, see Auth and
-     * the callback asks for a logmessage.
+     * the callback asks for a logmessage. With subversion 1.4 the target
+     * must not exist (\sa svn_client_move4)
      *
      * @param path
      * @param message log message. if it is QString::null asks when working on repository
@@ -429,6 +435,22 @@ namespace svn
     import (const Path & path, const QString& url,
             const QString& message,
             bool recurse) throw (ClientException)=0;
+    /**
+     * Import file or directory PATH into repository directory URL at
+     * head.  This usually requires authentication, see Auth.
+     * @param path path to import
+     * @param url
+     * @param message log message.
+     * @param recurse do it recursive?
+     * @param no_ignore if false, don't add items matching global ignore pattern
+     * @since subversion 1.3
+     * @exception ClientException
+     */
+    virtual void
+    import (const Path & path, const QString& url,
+            const QString& message,
+            bool recurse,
+            bool no_ignore) throw (ClientException)=0;
 
     /**
      * Merge changes from two paths into a new local path.
