@@ -1,3 +1,8 @@
+/*
+ * Port for usage with qt-framework and development for kdesvn
+ * (C) 2005-2007 by Rajko Albrecht
+ * http://www.alwins-world.de/wiki/programs/kdesvn
+ */
 /*
  * ====================================================================
  * Copyright (c) 2002-2005 The RapidSvn Group.  All rights reserved.
@@ -36,6 +41,7 @@
 #include "svnqt/url.hpp"
 #include "svnqt/svnqt_defines.hpp"
 
+#include <qurl.h>
 
 namespace svn
 {
@@ -69,6 +75,26 @@ namespace svn
         }
       }
       m_path = QString::FROMUTF8(int_path);
+#if QT_VERSION < 0x040000
+      if (Url::isValid(path) && m_path.find("@")!=-1 ) {
+#else
+      if (Url::isValid(path) && m_path.indexOf("@")!=-1 ) {
+#endif
+        /// @todo make sure that "@" is never used as revision paramter
+        QUrl uri = m_path;
+        m_path = uri.path();
+        m_path.replace("@","%40");
+#if QT_VERSION < 0x040000
+        m_path = uri.protocol()+"://"+(uri.hasUser()?uri.user()+(uri.hasPassword()?":"+uri.password():"")+"@":"")
+                +uri.host()+m_path;
+#else
+        m_path = uri.scheme()+"://"+uri.authority()+m_path;
+#endif
+        if (m_path.endsWith("/")) {
+            int_path = svn_path_internal_style (path.TOUTF8(), pool.pool () );
+            m_path = QString::FROMUTF8(int_path);
+        }
+      }
     }
   }
 
@@ -90,7 +116,9 @@ namespace svn
     }
     Pool pool;
     const char * int_path = svn_path_uri_decode(m_path.TOUTF8(), pool.pool () );
-    return QString::FROMUTF8(int_path);
+    QString _p = QString::FROMUTF8(int_path);
+    _p.replace("%40","@");
+    return _p;
   }
 
   const QByteArray
@@ -117,7 +145,7 @@ namespace svn
   void
   Path::addComponent (const QString& component)
   {
-    Pool pool;
+      Pool pool;
 
     if (Url::isValid (m_path))
     {
