@@ -1,3 +1,8 @@
+/*
+ * Port for usage with qt-framework and development for kdesvn
+ * (C) 2005-2007 by Rajko Albrecht
+ * http://www.alwins-world.de/wiki/programs/kdesvn
+ */
 /*
  * ====================================================================
  * Copyright (c) 2002-2005 The RapidSvn Group.  All rights reserved.
@@ -241,6 +246,13 @@ namespace svn
       return m_Data->m_Lock.Locked();
     }
 
+    const bool
+    Status::isModified()const
+    {
+        return textStatus()==svn_wc_status_modified||propStatus()==svn_wc_status_modified
+                ||textStatus ()==svn_wc_status_replaced;
+    }
+
     const bool
     Status::isRealVersioned()const
     {
